@@ -1,8 +1,7 @@
-use anyhow::Result;
-use async_std::fs::File;
+use anyhow::{anyhow, Context, Result};
 use biliup::client::{Client, LoginInfo};
-use biliup::line::{Line, Probe};
-use biliup::video::{Studio, Video};
+use biliup::line::Probe;
+use biliup::video::{BiliBili, Studio, Video};
 use biliup::{line, load_config};
 use clap::{IntoApp, Parser, Subcommand};
 use dialoguer::theme::ColorfulTheme;
@@ -13,7 +12,7 @@ use indicatif::{ProgressBar, ProgressStyle};
 use qrcode::render::unicode;
 use qrcode::QrCode;
 use std::ffi::OsStr;
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 use std::time::Instant;
 
 #[derive(Parser)]
@@ -35,7 +34,7 @@ enum Commands {
     Upload {
         // Optional name to operate on
         // name: Option<String>,
-        /// 需要上传的视频路径
+        /// 需要上传的视频路径,若指定配置文件投稿不需要此参数
         #[clap(parse(from_os_str))]
         video_path: Vec<PathBuf>,
 
@@ -46,6 +45,13 @@ enum Commands {
         /// 选择上传线路，支持kodo, bda2, qn, ws
         #[clap(short, long)]
         line: Option<String>,
+
+        /// 单视频文件最大并发数
+        #[clap(long, default_value = "3")]
+        limit: usize,
+
+        #[clap(flatten)]
+        studio: Studio,
     },
 }
 
@@ -73,7 +79,7 @@ pub async fn parse() -> Result<()> {
     // You can check for the existence of subcommands, and if found use their
     // matches just as you would the top level app
     let client: Client = Default::default();
-    match &cli.command {
+    match cli.command {
         Commands::Login => {
             login(client).await?;
         }
@@ -81,36 +87,46 @@ pub async fn parse() -> Result<()> {
             video_path,
             config: None,
             line,
-        } if video_path.len() > 0 => {
+            limit,
+            mut studio,
+        } if !video_path.is_empty() => {
+            println!("number of concurrent futures: {limit}");
             let login_info = client
                 .login_by_cookies(std::fs::File::open("cookies.json")?)
                 .await?;
-            let studio: Studio = Studio::builder()
-                .title(
-                    video_path[0]
-                        .file_stem()
-                        .and_then(OsStr::to_str)
-                        .map(|s| s.to_string())
-                        .unwrap(),
-                )
-                .videos(upload(video_path, &client, line.as_deref()).await?)
-                .build();
+            if studio.title.is_empty() {
+                studio.title = video_path[0]
+                    .file_stem()
+                    .and_then(OsStr::to_str)
+                    .map(|s| s.to_string())
+                    .unwrap();
+            }
+            cover_up(&mut studio, &login_info, &client).await?;
+            studio.videos = upload(&video_path, &client, line.as_deref(), limit).await?;
             studio.submit(&login_info).await?;
         }
         Commands::Upload {
-            video_path,
+            video_path: _,
             config: Some(config),
             ..
         } => {
             let login_info = client
                 .login_by_cookies(std::fs::File::open("cookies.json")?)
                 .await?;
-            for (prefix_filename, mut studio) in load_config(config)?.streamers {
+            let config = load_config(&config)?;
+            println!("number of concurrent futures: {}", config.limit);
+            for (filename_patterns, mut studio) in config.streamers {
                 let mut paths = Vec::new();
-                for entry in glob::glob(&format!("./{prefix_filename}*"))?.filter_map(Result::ok) {
+                for entry in glob::glob(&format!("./{filename_patterns}"))?.filter_map(Result::ok) {
                     paths.push(entry);
                 }
-                studio.videos = upload(&paths, &client, None).await?;
+                if paths.is_empty() {
+                    println!("未搜索到匹配的视频文件：{filename_patterns}");
+                    continue;
+                }
+                cover_up(&mut studio, &login_info, &client).await?;
+                studio.videos =
+                    upload(&paths, &client, config.line.as_deref(), config.limit).await?;
                 studio.submit(&login_info).await?;
             }
         }
@@ -129,13 +145,32 @@ async fn login(client: Client) -> Result<()> {
         .item("账号密码")
         .item("短信登录")
         .item("扫码登录")
+        .item("浏览器登录")
         .interact()?;
-    match selection {
+    let info = match selection {
         0 => login_by_password(client).await?,
         1 => login_by_sms(client).await?,
         2 => login_by_qrcode(client).await?,
+        3 => login_by_browser(client).await?,
         _ => panic!(),
     };
+    let file = std::fs::File::create("cookies.json")?;
+    serde_json::to_writer_pretty(&file, &info)?;
+    println!("登录成功，数据保存在{:?}", file);
+    Ok(())
+}
+
+async fn cover_up(studio: &mut Studio, login_info: &LoginInfo, client: &Client) -> Result<()> {
+    if !studio.cover.is_empty() {
+        let url = BiliBili::new(login_info, client)
+            .cover_up(
+                &std::fs::read(Path::new(&studio.cover))
+                    .with_context(|| format!("cover: {}", studio.cover))?,
+            )
+            .await?;
+        println!("{url}");
+        studio.cover = url;
+    }
     Ok(())
 }
 
@@ -143,6 +178,7 @@ pub async fn upload(
     video_path: &[PathBuf],
     client: &Client,
     line: Option<&str>,
+    limit: usize,
 ) -> Result<Vec<Video>> {
     let mut videos = Vec::new();
     let line = match line {
@@ -150,8 +186,8 @@ pub async fn upload(
         Some("bda2") => line::bda2(),
         Some("ws") => line::ws(),
         Some("qn") => line::qn(),
+        Some(name) => panic!("不正确的线路{name}"),
         None => Probe::probe().await.unwrap_or_default(),
-        _ => panic!("不正确的线路"),
     };
     // let line = line::kodo();
     for video_path in video_path {
@@ -167,7 +203,7 @@ pub async fn upload(
 
         let instant = Instant::now();
         let video = uploader
-            .upload(&client, |len| {
+            .upload(client, limit, |len| {
                 uploaded += len;
                 pb.set_position(uploaded as u64);
                 true
@@ -190,8 +226,7 @@ pub async fn login_by_password(client: Client) -> Result<LoginInfo> {
     let password: String = Input::with_theme(&ColorfulTheme::default())
         .with_prompt("请输入密码")
         .interact()?;
-    let res = client.login_by_password(&username, &password).await?;
-    Ok(res)
+    client.login_by_password(&username, &password).await
 }
 
 pub async fn login_by_sms(client: Client) -> Result<LoginInfo> {
@@ -207,8 +242,7 @@ pub async fn login_by_sms(client: Client) -> Result<LoginInfo> {
         .with_prompt("请输入验证码")
         .interact_text()?;
     // println!("{}", payload);
-    let ret = client.login_by_sms(input, res).await?;
-    Ok(ret)
+    client.login_by_sms(input, res).await
 }
 
 pub async fn login_by_qrcode(client: Client) -> Result<LoginInfo> {
@@ -227,3 +261,13 @@ pub async fn login_by_qrcode(client: Client) -> Result<LoginInfo> {
     image.save("qrcode.png").unwrap();
     client.login_by_qrcode(value).await
 }
+
+pub async fn login_by_browser(client: Client) -> Result<LoginInfo> {
+    let value = client.get_qrcode().await?;
+    println!(
+        "{}",
+        value["data"]["url"].as_str().ok_or(anyhow!("{}", value))?
+    );
+    println!("请复制此链接至浏览器中完成登录");
+    client.login_by_qrcode(value).await
+}
